/* Copyright (c) 2009-2010 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

/**
 * Here is object that holds some basic utility methods.
 *
 * @type Object
 */
var tbNotesUtils =
{
  
	/**
	 * Helper function that returns current date and time (e.g. "2009-09-16 14:33:00").
	 * 
	 * @returns string
	 */
	getCurrentDatetime: function() 
	{
		var d = new Date();
		var r = d.getFullYear() + "-" + 
						((d.getMonth()<10) ? "0" + d.getMonth() : d.getMonth()) + "-" + 
						((d.getDate()<10) ? "0" + d.getDate() : d.getDate()) + " " + 
						((d.getHours()<10) ? "0" + d.getHours() : d.getHours()) + ":" + 
						((d.getMinutes()<10) ? "0" + d.getMinutes() : d.getMinutes()) + ":00";
		
		return r;
	}, // end getCurrentDatetime()
	
	/**
	 * Removes all child nodes of given element.
	 * 
	 * @param aElm DOMElement
	 * @returns void
	 */
	removeChilds: function(aElm)
	{
		try {
			var ch = aElm.lastChild;
			while(ch) {
				aElm.removeChild(ch);
				ch = aElm.lastChild;
			}
		} catch(e) {
			Components.utils.reportError(e);
		}
	}, // end removeChilds(aElm)

	/**
	 * Copy given text to the system clipboard
	 *
	 * @param aText {string}
	 * @returns {boolean}
	 */
	copyTextToClipboard: function(aText)
	{
		try {
			let clip = Components.classes["@mozilla.org/widget/clipboard;1"].
					createInstance(Components.interfaces.nsIClipboard);
			let trans = Components.classes["@mozilla.org/widget/transferable;1"].
					createInstance(Components.interfaces.nsITransferable);
			let str = Components.classes["@mozilla.org/supports-string;1"].
					createInstance(Components.interfaces.nsISupportsString);
			
			str.data = aText;
			trans.addDataFlavor("text/unicode");
			trans.setTransferData("text/unicode", str, aText.length * 2);
			clip.setData(trans, null, Components.interfaces.nsIClipboard.kGlobalClipboard);

			return true;
		} catch(e) {
			Components.utils.reportError(e);
			
			return false;
		}
	}, // end copyTextToClipboard(aText)
	
	// TODO Implement another method for opening links (according to user's preferences)
	openLinkInInternalBrowser: function(aUrl)
	{
		// ...
	}, // end openLinkInInternalBrowser(aUrl)
	
	/**
	 * Opens link in the system browser.
	 *
	 * @param aUrl {string}
	 * @returns {void}
	 */
	openLinkInExternalBrowser: function(aUrl)
	{
		let ios = Components.classes["@mozilla.org/network/io-service;1"].
				getService(Components.interfaces.nsIIOService);
		let uri = ios.newURI(aUrl, null, null);
		let extps = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"].
				getService(Components.interfaces.nsIExternalProtocolService);

		extps.loadURI(uri, null);
	}, // end openLinkInExternalBrowser(aUrl),
	
	/**
	 * Returns nsIFile of the note template of given type.
	 * 
	 * @param aType {string} Type of template [html|text].
	 * @returns {Components.interfaces.nsIFile}
	 */
	getNoteTemplateFile: function(aType)
	{
		let filename = "temp/note-template." + ((aType == "text" || aType == "txt") ? "txt" : "html");
		
		let eid = "tbnotes@ondrejd.info";
		let em = Components.classes["@mozilla.org/extensions/manager;1"].
				getService(Components.interfaces.nsIExtensionManager);
		let file = em.getInstallLocation(eid).getItemFile(eid, filename);
		
		return file;
	}, // end getNoteTemplateFile(aType)
	
	/**
	 * Returns nsIUri from the path of given file
	 * 
	 * @param aType {Components.interfaces.nsIFile}
	 * @returns {Components.interfaces.nsIUri}
	 */
	getUriFromFile: function(aFile)
	{
		let ios = Components.classes["@mozilla.org/network/io-service;1"].
				getService(Components.interfaces.nsIIOService);
		let uri = ios.newFileURI(aFile);
		
		return uri;
	}, // end getUriFromFile(aFile)
	
	/**
	 * Returns nsIUri to the note template of given type.
	 * 
	 * @param aType {string} Type of template [html|text].
	 * @returns {Components.interfaces.nsIUri}
	 */
	getNoteTemplateFileUri: function(aType)
	{
		let file = this.getNoteTemplateFile(aType);
		let uri = this.getUriFromFile(file);
		
		return uri;
	}, // end getNoteTemplateFileUri(aType)
	
	/**
	 * Creates temporary text or html file from the given note.
	 * 
	 * @param aNote {tbNotesNotePrototype}
	 * @param aType {string} Type of file [html|text].
	 * @returns {Components.interfaces.nsIFile}
	 */
	createTempFileForNote: function(aNote, aType)
	{
		let title = new String(aNote.mTitle);
		title = title.toLowerCase();
		title = title.replace(" ", "-");
		let fileext = (aType == "text" || aType == "txt") ? "txt" : "html";
		let filename = title + "." + fileext;
		
		let eid = "tbnotes@ondrejd.info";
		let em = Components.classes["@mozilla.org/extensions/manager;1"].
				getService(Components.interfaces.nsIExtensionManager);
		let file = em.getInstallLocation(eid).getItemFile(eid, "temp");
		file.append(filename);
		
		let fos = Components.classes["@mozilla.org/network/file-output-stream;1"].
				createInstance(Components.interfaces.nsIFileOutputStream);
		fos.init(file, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate

		let conv = Components.classes["@mozilla.org/intl/converter-output-stream;1"].
				createInstance(Components.interfaces.nsIConverterOutputStream);
		conv.init(fos, "UTF-8", 0, 0);
		conv.writeString(aNote.mBody);
		conv.close(); // this closes foStream
		
		return file;
	}, // edn createTempFileForNote(aNote, aType)
	
	removeTempFiles: function()
	{
		Components.utils.reportError('not finished yet!');
	}
	
}; // End of tbNotesUtils
